function [ data_nc ] = ssdb_convertArtsToNc(artsSSP, artsMeta, miscMeta,...
  log_data)
%CONVERTDATA_ARTSTONC Converts data from arts format to supported
%.nc-formt. Requires additional meta-data in miscMeta and log_data.
%
% Length of frequency and temperature fields must be one.
%
% FORMAT [ data_nc ] = ssdb_convertArtsToNc(artsSSP, artsMeta, miscMeta,...
%  log_data)
%
% IN
%   artsSsp     ARTS ssp data struct.
%   artsMeta    ARTS meta data struct.
%   miscMeta    Struct containing extra needed meta data.
%   log_data    Struct containing ADDA log data.
%
% OUT
%   data_nc     Output struct.
%                   Fields:
%                       SingleScatteringData
%                       ScatteringMetaData
%                       MiscMetaData
%
% 2017-04-07 Robin Ekelund
% 2017-04-19 Robin Ekelund: Defined error, empty grids not allowed.
% 2020-12-14 RObin Ekelund: Fixed wrong dimensions for azimuthal ori.

%% ssp
data_nc.SingleScatteringData.frequency = artsSSP.f_grid;
data_nc.SingleScatteringData.temperature= artsSSP.T_grid;
data_nc.SingleScatteringData.aa_scat = artsSSP.aa_grid;
data_nc.SingleScatteringData.za_scat = artsSSP.za_grid;
switch artsSSP.ptype
    case 'totally_random'
        data_nc.SingleScatteringData.aa_inc = 0;
        data_nc.SingleScatteringData.za_inc = 0;
        n_phaElem = 6;
        n_extElem = 1;
        n_absElem = 1;

        phaMat_index = [1 2 0 0; 2 3 0 0; 0 0 4 5; 0 0 -5 6];
        extMat_index = [1 0 0 0; 0 1 0 0; 0 0 1 0; 0 0 0 1];
        absVec_index = [1 0 0 0]';

        data_nc.SingleScatteringData.orient_type = 'totally_random';
    case 'azimuthally_random'
        data_nc.SingleScatteringData.aa_inc = 0;
        data_nc.SingleScatteringData.za_inc = artsSSP.za_grid;
        n_phaElem = 16;
        n_extElem = 3;
        n_absElem = 2;

        phaMat_index = [1 2 3 4; 5 6 7 8; 9 10 11 12; 13 14 15 16];
        extMat_index = [1 2 0 0; 2 1 0 0; 0 0 1 3; 0 0 -3 1];
        absVec_index = [1 2 0 0]';

        data_nc.SingleScatteringData.orient_type = 'azimuthally_random';
    otherwise
        error('convertData_ArtsToNc:UnsupportedOption', 'Given ptype not supported.')
end
data_nc.SingleScatteringData.phaMat_index = phaMat_index;
data_nc.SingleScatteringData.extMat_index = extMat_index;
data_nc.SingleScatteringData.absVec_index = absVec_index;

n_freq = length(data_nc.SingleScatteringData.frequency);
n_T = length(data_nc.SingleScatteringData.temperature);
if n_T ~= 1
    error('convertData_ArtsToNc:BadInput', 'Temperature vector not length 1.')
end
if n_freq ~= 1
    error('convertData_ArtsToNc:BadInput', 'Frequency vector not length 1.')
end

n_aa_inc = length(data_nc.SingleScatteringData.aa_inc);
n_za_inc = length(data_nc.SingleScatteringData.za_inc);

n_aa_scat = length(data_nc.SingleScatteringData.aa_scat);
n_za_scat = length(data_nc.SingleScatteringData.za_scat);

if any([n_aa_inc n_za_inc n_aa_scat n_za_scat] == 0)
    error('convertData_ArtsToNc:BadInput', 'Empty grids not allowed')
end

data_nc.SingleScatteringData.phaMat_data = reshape(artsSSP.pha_mat_data, [n_za_scat, n_aa_scat, n_za_inc, n_aa_inc, n_phaElem]);
data_nc.SingleScatteringData.extMat_data = reshape(artsSSP.ext_mat_data, [n_za_inc, n_aa_inc, n_extElem]);
data_nc.SingleScatteringData.absVec_data = reshape(artsSSP.abs_vec_data, [n_za_inc, n_aa_inc, n_absElem]);

% shape
data_nc.ShapeData.description = artsMeta.description;
data_nc.ShapeData.source = artsMeta.source;
data_nc.ShapeData.refrIndex_model = artsMeta.refr_index;
data_nc.ShapeData.habit_file_id = miscMeta.habit_name;
data_nc.ShapeData.habit_id = miscMeta.habit_id;
data_nc.ShapeData.shape_file = miscMeta.shape_file;
data_nc.ShapeData.phase = miscMeta.phase;
data_nc.ShapeData.refrIndex_homogenous_bool = miscMeta.refrIndex_homogenous_bool;
data_nc.ShapeData.density_homogenous_bool = miscMeta.density_homogenous_bool;
data_nc.ShapeData.diameter_max = artsMeta.diameter_max;
data_nc.ShapeData.diameter_vol_eq = artsMeta.diameter_volume_equ;
data_nc.ShapeData.diameter_area_eq_aerodynamical = artsMeta.diameter_area_equ_aerodynamical;
data_nc.ShapeData.mass = artsMeta.mass;
data_nc.ShapeData.aspect_ratio = miscMeta.aspect_ratio;
data_nc.ShapeData.dpl = miscMeta.dpl;
data_nc.ShapeData.N_dipoles = miscMeta.N_dp;
data_nc.ShapeData.refrIndex_real = real(miscMeta.refrIndex);
data_nc.ShapeData.refrIndex_imag = imag(miscMeta.refrIndex);
data_nc.ShapeData.alpha = miscMeta.alpha;
data_nc.ShapeData.beta = miscMeta.beta;
data_nc.ShapeData.gamma = miscMeta.gamma;

% method
data_nc.CalculationData.method = 'DDA';
data_nc.CalculationData.software = 'ADDA';
data_nc.CalculationData.software_version = log_data.software_version;
data_nc.CalculationData.system = log_data.system;
data_nc.CalculationData.n_nodes = log_data.N_nodes;
data_nc.CalculationData.n_cores = log_data.N_cores;
data_nc.CalculationData.date_completion = log_data.date;
data_nc.CalculationData.ADDA_eps = log_data.eps;
data_nc.CalculationData.ADDA_avgParam_file = log_data.ADDA_avgParam_file;
data_nc.CalculationData.ADDA_scatParam_file = log_data.ADDA_scatParam_file;
end

