function [ db_def ] = ssdb_fileStructDef( habit_param, topFolder, orient_type )
%ssdb_fileStructDef Returns database file structure definition for given
% input habit.
%
% FORMAT [ db_def ] = ssdb_fileStructDef( habit_param, topFolder,...
% orient_type )
%
% IN
%   habit_param     Struct containing habit data (see ssdb_habitSpec). 
%                       habit name or habit id is also accpeted if supported
%                       in ssdb_habisSpec.
%   topFolder       The top folder of the database.
%   orient_type     Orientation type (e.g., 'totally_random' or
%                       'azimuthally_random')
%
% OUT
%   db_def          Struct containing database file structure definition.
%
% 2017-04-07 Robin Ekelund
% 2017-04-07 Robin Ekelund: Fixed summary.txt definition
% 2017-10-12 Robin Ekelund: Restructurning and changing the folder
%                               structure.
% 2018-05-22 Robin Ekelund: Changed folder name format for ADDA data. Added
%                               ID field.
%
%% Definitions
% Allowed folder names
phase_map = containers.Map({'ice' 'mixed' 'liquid'},...
    {'Ice' 'Mixed' 'Liquid'});
macroType_map = containers.Map({'aggregate' 'singleCrystal'},...
    {'Aggregates' 'SingleCrystals'});
microType_map = containers.Map({'pristine' 'rimed'},...
    {'Pristine' 'Rimed'});
orientType_map_top = containers.Map({'totally_random' 'azimuthally_random'},...
    {'TotallyRandom' 'AzimuthallyRandom'});
orientType_map_bottom = containers.Map({'totally_random' 'azimuthally_random'},...
    {'TotallyRandom' 'AzimuthallyRandom_beta%05.1fdeg'});

% file ids
file_id_adda = '.adda';
file_id_metaMat = '.meta.mat';
file_id_nc = '.nc';

%% Setup
db_def = struct('ID', [], 'shape_path', [], 'ssd_path', [],...
    'rawData_path', []);
if exist('habit_param', 'var') ~= 1 || isempty(habit_param)
    return
end

if ischar(habit_param) || isnumeric(habit_param)
    habit_param = ssdb_habitSpec(habit_param);
end

%% Unique job IDs
temp.format = {''};
temp.variables = {{}};
temp.varFactors = {[]};

temp = append_phaseFormat(temp, habit_param.phase, phase_map);
temp = append_macroFormat(temp, habit_param.macro_type, macroType_map);
temp = append_microFormat(temp, habit_param.micro_type, microType_map);
temp = append_habitFormat(temp, habit_param.name, habit_param.id);
temp = append_sizeFormat(temp);
temp = append_orientFormat_bottom(temp, orient_type, orientType_map_bottom);
db_def.ID.ID1 = append_freqTempFormat(temp);
db_def.ID.ID2 = append_freqTempEpsFormat(temp);
if isempty(topFolder)
   return 
end

%% Shape def
clear temp
input_folder = 'Input';
shape_folder = 'Shapes';
temp.format = {topFolder input_folder shape_folder};
temp.variables = {{} {} {}};
temp.varFactors = {[] [] []};

temp = append_phaseFormat(temp, habit_param.phase, phase_map);
temp = append_macroFormat(temp, habit_param.macro_type, macroType_map);
temp = append_microFormat(temp, habit_param.micro_type, microType_map);
temp = append_habitFormat(temp, habit_param.name, habit_param.id);
temp = append_sizeShapeFormat(temp);

db_def.shape_path.adda = append_fileId(temp, file_id_adda);
db_def.shape_path.metaMat = append_fileId(temp, file_id_metaMat);

%% SSD def
ssd_folder = 'SSD';
temp.format = {topFolder ssd_folder};
temp.variables = {{} {}};
temp.varFactors = {[] []};

temp = append_orientFormat_top(temp, orient_type, orientType_map_top);
temp = append_phaseFormat(temp, habit_param.phase, phase_map);
temp = append_macroFormat(temp, habit_param.macro_type, macroType_map);
temp = append_microFormat(temp, habit_param.micro_type, microType_map);
temp = append_habitFormat(temp, habit_param.name_id, habit_param.id);

db_def.ssd_path.top_folder = temp;
db_def.ssd_path.report = append_struct(temp, {'report.pdf'}, {{}}, {[]});
db_def.ssd_path.summary_path = append_struct(temp, {'DataSummary.txt'}, {{}}, {[]});
db_def.ssd_path.img_path = append_struct(temp, {'shape_img.png'}, {{}}, {[]});

temp = append_orientFormat_bottom(temp, orient_type, orientType_map_bottom);
db_def.ssd_path.bottom_folder = temp;
temp = append_sizeFormat(temp);
temp = append_fileId(temp, file_id_nc);

db_def.ssd_path.nc_file = temp;
db_def.ssd_path.nc_data = append_freqTempFormat(temp);

%% ADDA raw output
ssd_folder = 'SSD';
temp.format = {topFolder ssd_folder};
temp.variables = {{} {}};
temp.varFactors = {[] []};

temp = append_phaseFormat(temp, habit_param.phase, phase_map);
temp = append_macroFormat(temp, habit_param.macro_type, macroType_map);
temp = append_microFormat(temp, habit_param.micro_type, microType_map);
temp = append_habitFormat(temp, habit_param.name, habit_param.id);
temp = append_sizeFormat(temp);
temp = append_orientFormat_bottom(temp, orient_type, orientType_map_bottom);
db_def.rawData_path.bottom_folder_old = append_freqTempFormat(temp);
db_def.rawData_path.bottom_folder = append_freqTempEpsFormat(temp);
db_def.rawData_path.bottom_folder_ver = append_freqTempVerFormat(temp);
end

function [prm_struct] = append_habitFormat(prm_struct, habit_name, habit_id)
    format = sprintf('%s_Id%d', habit_name, habit_id);
    variables = {{}};
    varFactors = {[]};
    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_phaseFormat(prm_struct, phase, phase_map)
    if ismember(phase, keys(phase_map))
        format = phase_map(phase);
    else
        error('Phase not supported, was %s', habit_param.phase)
    end
    variables = {{}};
    varFactors = {[]};
    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_macroFormat(prm_struct, macro_type, macro_map)
    if ismember(macro_type, keys(macro_map))
        format = macro_map(macro_type);
    else
        error('macro_type not supported, was %s', habit_param.macro_type)
    end
    variables = {{}};
    varFactors = {[]};
    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_microFormat(prm_struct, micro_type, micro_map)
    if ismember(micro_type, keys(micro_map))
        format = micro_map(micro_type);
    else
        error('micro_type not supported, was %s', habit_param.micro_type)
    end
    variables = {{}};
    varFactors = {[]};
    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_struct(prm_struct, format, variables, varFactors)
    prm_struct.format = [prm_struct.format format];
    prm_struct.variables = [prm_struct.variables variables];
    prm_struct.varFactors = [prm_struct.varFactors varFactors];
end

function [prm_struct] = append_freqTempFormat(prm_struct)
    variables = {{'frequency' 'temperature'}};
    varFactors =  {[1e9 1]};
    format = 'Freq%08.3fGHz_T%06.2fK';
    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_freqTempEpsFormat(prm_struct)
    variables = {{'frequency' 'temperature' 'eps' 'eps_avg'}};
    varFactors =  {[1e9 1 1 1]};
    format = 'Freq%08.3fGHz_T%06.2fK_eps%02.0f_epsAvg%02.0f';
    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_freqTempVerFormat(prm_struct)
    variables = {{'frequency' 'temperature' 'version'}};
    varFactors =  {[1e9 1 1]};
    format = 'Freq%08.3fGHz_T%06.2fK_v%01.0f';
    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_orientFormat_bottom(prm_struct, orient_type,...
    orient_map)
    if strcmp(orient_type, 'totally_random')
        variables = {{}};
        varFactors = {[]};
    elseif strcmp(orient_type, 'azimuthally_random')
        variables = {{'beta'}};
        varFactors = {1};
    end
    format = orient_map(orient_type);
    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_orientFormat_top(prm_struct, orient_type,...
    orient_map)
    if strcmp(orient_type, 'totally_random')
        variables = {{}};
        varFactors = {[]};
    elseif strcmp(orient_type, 'azimuthally_random')
        variables = {{}};
        varFactors = {[]};
    end
    format = orient_map(orient_type);
    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_fileId(prm_struct, file_ident)
    allowed_fileId = {'.adda' '.meta.mat' '.nc'};
    if ~ismember(file_ident ,allowed_fileId)
        error('file identifier not allowed, was %s', file_ident)
    end

    prm_struct.format{end} = [prm_struct.format{end} file_ident];
end

function [prm_struct] = append_sizeShapeFormat(prm_struct)
    variables = {{'diameter_max' 'diameter_vol_eq' 'mass' 'frequency_min'}};
    varFactors = {[1e-6 1e-6 1 1e9]};
    format = 'Dmax%05.0fum_Dveq%05.0fum_Mass%011.5ekg_Freq%08.3fGHz';

    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

function [prm_struct] = append_sizeFormat(prm_struct)
    variables = {{'diameter_max' 'diameter_vol_eq' 'mass' }};
    varFactors = {[1e-6 1e-6 1]};
    format = 'Dmax%05.0fum_Dveq%05.0fum_Mass%011.5ekg';

    prm_struct = append_struct(prm_struct, format, variables, varFactors);
end

