function [ path ] = ssdb_getFilePath( fileStructDef, param )
%SSDB_GETFILEPATH Returns path for given file structure definition and
%parametrization.
%
% Example usage:
%
% % get file structure
% file_def = ssdb_fileStructDef(habit_param, topFolder, orient_type);
%
% % define param
% prm.frequency_min = 180e9
% prm.mass = 7e-19
% prm.diameter_max = 10e-6
% prm.diameter_vol_eq = 18e-6
%
% % get path
% path = ssdb_getFilePath(file_def.shape_path.adda, prm);
%
%
% FORMAT [ path ] = ssdb_getFilePath( fileStructDef, param )
%
% IN
%   fileStructDef   Structure that defines the folder and file format. Is
%                       retrieved using ssdb_fileStructDef.
%   param           Struct which fields contain the necessary parameters
%                       for defining the output path. Required fields
%                       depends on what fileStructDef needs. Error will be
%                       raised if a field is missing.
%
% OUT
%   path            path
%
% 2017-04-07 Robin Ekelund
% 2017-10-23 Robin Ekelund : l.65, Added d_veq
% 2017-10-31 Robin Ekelund : l.70, Added temp
% 2018-02-13 Robin Ekelund : l.73, Added ar_cryst
% 2018-10-29 Robin Ekelund : Added eps_avg as possible variable.
%
%% Extract data
format = fileStructDef.format;
variables = fileStructDef.variables;
varFactors = fileStructDef.varFactors;
%% Create path
path = '';
for i=1:length(variables)
    str_ident = format{i};
    varNames = variables{i};
    varFac = varFactors{i};

    if ~iscell(varNames)
        varNames = {varNames};
    end
    N = length(varNames);
    if isempty(varFac)
        varFac = ones(1, N);
    end
    var = cell(1, N);
    for n=1:N
        val = get_paramVal(varNames{n},param);
        if isnumeric(val)
            var{n} = val/varFac(n);
        elseif ischar(val)
            var{n} = val;
        else
            error('IE')
        end
    end

    temp = sprintf(str_ident, var{:});

    path = fullfile(path, temp);
end
end

function [val] = get_paramVal(varName, param)
    if ~isfield(param, varName)
        switch varName
            case 'frequency_min'
                varOpt = {'freq_min'};
            case 'diameter_max'
                varOpt = {'d_max'};
            case 'diameter_vol_eq'
                varOpt = {'d_eq' 'd_veq'};
            case 'frequency'
                varOpt = {'freq'};
            case 'temperature'
                varOpt = {'T' 'temp'};
            case 'ar_cryst'
                varOpt = {'aspect_ratio_cryst'};
            case 'id'
                varOpt = {'ID'};
            case 'eps_avg'
                varOpt = {'eps_avg_val'};
            otherwise
                error('getFilePath:MiscError', 'No alternatives to variable name ''%s'' are recognized', varName);
        end
        I = isfield(param, varOpt);
        if sum(I) == 0
            error('getFilePath:MiscError', 'Variable name ''%s'' not found in param', varName);
        end
        varName = varOpt{I};
    end
    val = param.(varName);
end

