function [ habit_param ] = ssdb_habitSpec( type )
%SSDB_HABITSPEC Returns database habit definitions
%
% Omitt input to get all habits.
%
% FORMAT [ habit_param ] = ssdb_habitSpec( type )
%
% OUT
%   habit_spec  All habit specifications (struct)
%
% IN
%   type        Either id or name of habit
%
% OUT
%   habit_param  Habit specifications (struct)
%
%%
habit_param = get_habit();
if exist('type', 'var') ~= 1 || isempty(type)
    I = 1:length(habit_param);
elseif isnumeric(type)
    I = ismember([habit_param.id], type);
elseif ischar(type) || iscellstr(type)
    I = ismember({habit_param.name}, type);
else
    error('ssdb_habitSpec:InputTypeBad', 'Input type not recognized, must be numeric, string or cell of strings.')
end
if sum(I) == 0
    error('ssdb_habitSpec:InputNotSupported', ['Input not found, was "' num2str(type) '"'])
else
    habit_param = habit_param(I);
end
end

function [habit_param] = get_habit()
    % Returns supported habits
    habit_param = struct('name', {}, 'id', {}, 'phase', {},...
        'macro_type', {}, 'micro_type', {},...
        'refrIndex_homogenous_bool', {}, 'density_homogenous_bool', {},...
        'source', {}, 'description', {}, 'calculation_type', {});
   id = 0;
    while true
        % Default values
        refrIndex_homogenous_bool = 1;
        density_homogenous_bool = 1;
        source_chalmers_torbjorn = 'location: Chalmers, software: Snowflake-toolkit (Rathsman 2016).';
        source_chalmers_torbjorn_Liu = 'location: Chalmers, software: Snowflake-toolkit (Rathsman 2016), parameterization: Liu 2008.';
        source_chalmers_torbjorn_Hong = 'location: Chalmers, software: Snowflake-toolkit (Rathsman 2016), parameterization: Hong 2007.';
        source_mie = 'location: Chalmers';
        source_tmat = 'location: Chalmers';
        source_chalmers_rimecraft = 'location: Chalmers, software: Rimecraft.';
        
        calcType_adda = 'ADDA';
        calcType_mie = 'MIE';
        calcType_tmat = 'TMATRIX';

        id = id +1;
        
        % Assign values
        refrIndex_model_ice = 'Matzler 2006';
        refrIndex_model_liq = 'water_ellison07';
        refrIndex_model = refrIndex_model_ice;
        calcType = calcType_adda;
        switch id
            case 1
                name = 'EvansSnowAgg';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = 'Evans 2012.';
                description = 'Snowflake aggregate, shape files from Evans 2012';
                title_name = 'Evans Snow Aggregate';
                name_id = strrep(title_name, ' ', '');
                comment = '';
            case 2
                name = 'HongBulletRosette-5b';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Hong;
                description = 'Hexagonal 5-bullet rosette. The bullets have the same dimensions as in the Hong bullet rosette (Hong 2007).';
                title_name = '5-bullet Rosette';
                name_id = strrep(title_name, ' ', '');
                comment = '-';
            case 3
                name = 'LiuSectorSnowflake';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Liu;
                description = 'Sector snowflake, using Liu parametrization (Liu 2008).';
                title_name = 'Sector Snowflake';
                name_id = strrep(title_name, ' ', '');
                comment = 'Note that the alpha and beta values are calculated for the upper end of d_max, since the shape transitions to spheroidal in the lower end of d_max.';
            case 4
                name = 'HongBulletRosette-3b-perp';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Hong;
                description = 'Perpendicular hexagonal 3-bullet rosette. The bullets have the same dimensions as in the Hong bullet rosette (Hong 2007).';
                title_name = 'Perpendicular 3-Bullet Rosette';
                name_id = strrep(title_name, ' ', '');
                comment = '-';
            case 5
                name = 'HongBulletRosette-3b-flat';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Hong;
                description = 'Flat hexagonal 3-bullet rosette. The bullets have the same dimensions as in the Hong bullet rosette (Hong 2007).';
                title_name = 'Flat 3-Bullet Rosette';
                name_id = strrep(title_name, ' ', '');
                comment = '-';
            case 6
                name = 'HongBulletRosette-6b';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Hong;
                description = 'Hexagonal 6-bullet rosette, using Hong parametrization (Hong 2007).';
                title_name = '6-Bullet Rosette';
                name_id = strrep(title_name, ' ', '');
                comment = '-';
            case 7
                name = 'HongColumn';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Hong;
                description = 'Hexagonal column, using Hong parametrization (Hong 2007).';
                title_name = 'Column Type 1';
                name_id = strrep(title_name, ' ', '');
                comment = 'Note that values of alpha and beta are calculated for the upper end of d_max.';
            case 8
                name = 'HongAggregate';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Hong;
                description = 'Hexagonal column aggregate, using Hong parametrization (Hong 2007).';
                title_name = '8-Column Aggregate';
                name_id = strrep(title_name, ' ', '');
                comment = 'Note that shape and proportions are constant with size, resulting in beta=3.';
            case 9
                name = 'HongPlate';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Hong;
                description = 'Hexagonal plate, using Hong parametrization (Hong 2007).';
                title_name = 'Plate Type 1';
                name_id = strrep(title_name, ' ', '');
                comment = 'Note that values of alpha and beta are calculated for the upper end of d_max.';
            case 10
                name = 'HongBulletRosette-4b-perp';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Hong;
                description = 'Perpendicular hexagonal 4-bullet rosette. The bullets have the same dimensions as in the Hong bullet rosette (Hong 2007).';
                title_name = 'Perpendicular 4-Bullet Rosette';
                name_id = strrep(title_name, ' ', '');
                comment = '-';
            case 11
                name = 'HongBulletRosette-4b-flat';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Hong;
                description = 'Flat hexagonal 4-bullet rosette. The bullets have the same dimensions as in the Hong bullet rosette (Hong 2007).';
                title_name = 'Flat 4-Bullet Rosette';
                name_id = strrep(title_name, ' ', '');
                comment = '-';
            case 12
                name = 'LiuBlockColumn';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Liu;
                description = 'Hexagonal block column, using Liu parametrization (Liu 2008).';
                title_name = 'Block Column';
                name_id = strrep(title_name, ' ', '');
                comment = 'Note that aspect ratio is constant.';
            case 13
                name = 'LiuShortColumn';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Liu;
                description = 'Hexagonal short column, using Liu parametrization (Liu 2008).';
                title_name = 'Short Column';
                name_id = strrep(title_name, ' ', '');
                comment = 'Note that aspect ratio is constant.';
            case 14
                name = 'LiuLongColumn';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Liu;
                description = 'Hexagonal long column, using Liu parametrization (Liu 2008).';
                title_name = 'Long Column';
                name_id = strrep(title_name, ' ', '');
                comment = 'Note that aspect ratio is constant.';
            case 15
                name = 'LiuThickPlate';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Liu;
                description = 'Hexagonal thick plate, using Liu parametrization (Liu 2008).';
                title_name = 'Thick Plate';
                name_id = strrep(title_name, ' ', '');
                comment = 'Note that aspect ratio is constant.';
            case 16
                name = 'LiuThinPlate';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn_Liu;
                description = 'Hexagonal thin plate, using Liu parametrization (Liu 2008).';
                title_name = 'Thin Plate';
                name_id = strrep(title_name, ' ', '');
                comment = 'Note that aspect ratio is constant.';
            case 17
                name = 'HexColAggCrystal-Dmax100um-AxisRatio5';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn;
                description = 'Hexagonal column aggregates, mean aspect ratio = 5. Prototype crystal d_max=100 um.';
                title_name = 'Small Column Aggregate';
                name_id = strrep(title_name, ' ', '');
                comment = '';
            case 18
                name = 'HexColAggCrystal-Dmax350um-AxisRatio5';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn;
                description = 'Hexagonal column aggregates, mean aspect ratio = 5. Prototype crystal d_max=350 um.';
                title_name = 'Large Column Aggregate';
                name_id = strrep(title_name, ' ', '');
                comment = '';
            case 19
                name = 'HexPlaAggCrystal-Dmax100um-AxisRatio6';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn;
                description = 'Hexagonal plate aggregates, mean aspect ratio = 6. Prototype crystal d_max=100 um.';
                title_name = 'Small Plate Aggregate';
                name_id = strrep(title_name, ' ', '');
                comment = '';
            case 20
                name = 'HexPlaAggCrystal-Dmax350um-AxisRatio6';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn;
                description = 'Hexagonal plate aggregates, mean aspect ratio = 6. Prototype crystal d_max=350 um.';
                title_name = 'Large Plate Aggregate';
                name_id = strrep(title_name, ' ', '');
                comment = '';
            case 21
                name = 'HexColAggCrystal-Dmax100um-AxisRatio1.25';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn;
                description = 'Hexagonal column aggregates, mean aspect ratio = 1.25. Prototype crystal d_max=100 um.';
                title_name = 'Small Block Aggregate';
                name_id = strrep(title_name, ' ', '');
                comment = '';
            case 22
                name = 'HexColAggCrystal-Dmax350m-AxisRatio1.25';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn;
                description = 'Hexagonal column aggregates, mean aspect ratio = 1.25. Prototype crystal d_max=350 um.';
                title_name = 'Large Block Aggregate';
                name_id = strrep(title_name, ' ', '');
                comment = '';
            case 23
                name = 'SphericalGraupel';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'rimed';
                source = source_chalmers_torbjorn;
                description = 'Spherical graupel';
                title_name = 'Spherical Graupel';
                name_id = strrep(title_name, ' ', '');
                comment = '';
            case 24
                name = 'IceSphere';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_mie;
                description = 'Ice sphere';
                title_name = 'Ice Sphere';
                comment = '';
                name_id = strrep(title_name, ' ', '');
                calcType = calcType_mie;
            case 25
                name = 'LiquidSphere';
                phase = 'liquid';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_mie;
                description = 'Liquid sphere';
                title_name = 'Liquid Sphere';
                comment = '';   
                refrIndex_model = refrIndex_model_liq;
                name_id = strrep(title_name, ' ', '');
                calcType = calcType_mie;
            case 26
                name = 'TyynelaFernDendAgg';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = 'Tyynela and Chandrasekar 2014.';
                description = 'Aggregate of fernlike dendrites.';
                title_name = 'Tyynela Dendrite Aggregate';
                name_id = strrep(title_name, ' ', '');
                comment = '';
            case 27
                name = 'IconCloudIce';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_rimecraft;
                description = 'ICON cloud ice';
                title_name = 'ICON Cloud Ice';
                name_id = name;
                comment = '';  
            case 28
                name = 'IconSnow';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_rimecraft;
                description = 'ICON snow';
                title_name = 'ICON Snow';
                name_id = name;
                comment = '';  
            case 29
                name = 'IconGraupel';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'rimed';
                source = source_chalmers_rimecraft;
                description = 'ICON graupel';
                title_name = 'ICON Graupel';
                name_id = name;
                comment = '';
            case 30
                name = 'IconHail';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_rimecraft;
                description = 'ICON hail';
                title_name = 'ICON Hail';
                name_id = name;
                comment = '';
            case 31
                name = 'GemCloudIce';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_rimecraft;
                calcType = calcType_adda;
                description = 'GEM cloud ice';
                title_name = 'GEM Cloud Ice';
                name_id = name;
                comment = '';      
            case 32
                name = 'GemSnow';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_rimecraft;
                description = 'GEM snow';
                title_name = 'GEM Snow';
                name_id = name;
                comment = '';  
            case 33
                name = 'GemGraupel';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'rimed';
                source = source_chalmers_rimecraft;
                description = 'GEM graupel';
                title_name = 'GEM Graupel';
                name_id = name;
                comment = '';
            case 34
                name = 'GemHail';
                phase = 'ice';
                macro_type = 'aggregate';
                micro_type = 'pristine';
                source = source_chalmers_rimecraft;
                description = 'GEM hail';
                title_name = 'GEM Hail';
                name_id = name;
                comment = '';
            case 35
                name = 'LiquidSpheroid-AREq2Ch';
                phase = 'liquid';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source =  'location: Chalmers, shape parameterization: Chuang and Beard (1990)';
                description = 'Liquid spheroid. Aspect ratio and mass are set to be equavalent to the CebyshevRainDrop habit (id 36).';
                title_name = 'Liquid Spheroid AREq2Ch';
                comment = 'See Ekelund et al. (2020) for more information.';   
                refrIndex_model = refrIndex_model_liq;
                name_id = strrep(title_name, ' ', '');
                calcType = calcType_tmat;
            case 36
                name = 'Rosette-6b-BrownFrancis';
                phase = 'ice';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_torbjorn;
                description = ['Hexagonal 6-column rosette, using '...
                    'Brown-Francis parametrization.'];
                title_name = 'Brown-Francis 6-Column Rosette';
                name_id = strrep(title_name, ' ', '');
                comment = '-';     
            case 37
                name = 'CebyshevRainDrop';
                phase = 'liquid';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = 'location: Chalmers, shape parameterization: Chuang and Beard (1990)';
                description = 'Chebyshev (Equilibrium) rain drop';
                title_name = 'Chebyshev Rain Drop';
                comment = 'See Ekelund et al. (2020) for more information.';   
                refrIndex_model = refrIndex_model_liq;
                name_id = strrep(title_name, ' ', '');
                calcType = calcType_tmat;
            case 38
                name = 'HongPlate-MeltFrac0.05';
                phase = 'mixed';
                macro_type = 'singleCrystal';
                micro_type = 'pristine';
                source = source_chalmers_rimecraft;
                description = 'Melting Hong plate, liquid fraction 0.05';
                title_name = 'Hong plate, melting fraction 0.05';
                comment = '';      
                refrIndex_homogenous_bool = 0;
                density_homogenous_bool = 0;
                refrIndex_model = {refrIndex_model_ice refrIndex_model_liq};
            otherwise
                break
        end
        name_id = strrep(name_id, '-', '');
        habit_param(id).name_id = name_id;
        habit_param(id).name = name;
        habit_param(id).id = id;
        habit_param(id).phase = phase;
        habit_param(id).macro_type = macro_type;
        habit_param(id).micro_type = micro_type;
        habit_param(id).source = source;
        habit_param(id).description = description;
        habit_param(id).title_name = title_name;
        habit_param(id).comment = comment;
        habit_param(id).refrIndex_model = refrIndex_model;
        habit_param(id).refrIndex_homogenous_bool =...
            refrIndex_homogenous_bool;
        habit_param(id).density_homogenous_bool =...
            density_homogenous_bool;
        habit_param(id).calculation_type = calcType;
        
        clear name phase macro_type micro_type ...
            refrIndex_homogenous_bool density_homogenous_bool source...
            description calcType
    end
end
