% WRITE_NCFILE   Writes single scattering data to a netCDF file. 
%
% The data is written to an internal netCDF structure. The structure is
% determined by input path. The function looks for an instance in path with
% identifier .nc, for which a netCDF-file is created or opened (if already
% existing). The internal netCDF structure is then determnied by the path
% lower in hierarchy.
%
% Multiple data instances can be written at once to the file in a cell.
% Corresponding paths must be given in a cell of same length. Note that the
% paths to the netcdf-file must be consistent in each path given (only one
% single file will be written to).
%
% The format of the written netCDF-file is defined by
%
% Note: nc_schema is stored in a persistent variable. It is overwritten if
%   nc_schema is given.
%
% FORMAT [ ] = assd_write_ncfile( path, data, db_version, nc_schema )
%
% IN
%   path        Path to netCDF file. Must contain one instance with
%                   .nc-identifier, which determines at what level the
%                   nc-files is created. Can be a cell for multiple data
%                   input.
%   data        Input data. Fields need to be the same as the attributes
%                   and variables defined in nc_schema. Must be a cell if
%                   multiple data is given.
%   db_version  Database version. Will be written into the nc-files on
%                   attribute "db_version".
% OPT
%   nc_schema   schema that describes the .nc-file format. Structure is as
%                   defined by the function ncinfo. If not given, the
%                   function will look for the orient_type field in the
%                   input data, and use that to set the format.
%
% 2016-11-03 Robin Ekelund: Created
% 2017-04-06 Robin Ekelund: Logical values are accepted. Cleanup.
% 2017-10-15 Robin Ekelund: Changed to write on an internal structure.
% 2018-02-13 Robin Ekelund: Added compression.
% 2020-10-11 Robin Ekelund: Added db_version as input.
%
function [ ] = ssdb_write_ncfile( path, data, db_version, nc_schema )
persistent nc_schema_p
N_data = length(data);
if isstruct(data) && N_data == 1
    data = {data};
end
if ischar(path) && N_data == 1
    path = {path};
end
if N_data ~= length(path)
    error('write_ncfile:BadInputData', 'Length of data and path inconsistent.')
end
if N_data > 1 && iscell(data) == 0
    error('write_ncfile:BadInputData', 'Multiple data input must be in the form of a cell.')
end
if N_data > 1 && iscell(path) == 0
    error('write_ncfile:BadInputData', 'Multiple path input must be in the form of a cell.')
end

if ~isempty(nc_schema_p) && strcmp(data{1}.SingleScatteringData.orient_type,...
      nc_schema_p.Groups(1).Attributes(1).Value) ~= 1
    nc_schema_p = [];
end
if exist('nc_schema_in', 'var') == 1
    nc_schema_p = nc_schema;
elseif isempty(nc_schema_p)
    if isfield(data{1}, 'SingleScatteringData') &&...
        isfield(data{1}.SingleScatteringData, 'orient_type')
        nc_schema_p =...
            ssdb_ncFormat(data{1}.SingleScatteringData.orient_type, db_version);

    else
        error('write_ncfile:BadInputData',...
            'orient_type not found in input data. Provide nc_schema explicitly to circumvent this.')
    end
end
%% Write data
% Set date
I = strcmp({nc_schema_p.Attributes.Name}, 'date');
nc_schema_p.Attributes(I).Value = date;

% Determine internal folder structure
folder_path = '';
for i=1:N_data
    data_tmp = data{i};
    path_tmp = path{i};

    % Determine path
    folders = strsplit(path_tmp, '/');
    level_nc = find(cellfun(@(x)isempty(strfind(x, '.nc'))==0, folders));
    if length(level_nc) ~= 1
        error('write_ncfile:BadInputData', 'Path string must contain .nc-identifier at one and only one folder level.')
    end
    folder_path_tmp = fullfile('/', folders{1:level_nc});
    if isempty(folder_path) == 1
       folder_path = folder_path_tmp;
    elseif strcmp(folder_path, folder_path_tmp) == 0
        error('write_ncfile:BadInputData', 'nc-file path is not consistent over all given paths')
    end
    folders_nc = folders(level_nc+1:end);

    % Create file or open existing file
    if exist('nc_id', 'var') == 0
        if exist(folder_path, 'file')
            nc_id = netcdf.open(folder_path, 'WRITE');
        else
            nc_id = netcdf.create(folder_path, nc_schema_p.Format);
        end
        cleanupObj = onCleanup(@()netcdf.close(nc_id));
    end

    % Set dimenions
    defineNc(nc_id, nc_schema_p, data_tmp, [], folders_nc);

    % Write data
    writeData(nc_id,  nc_schema_p, data_tmp, folders_nc);
end
end

function [] = writeData(nc_id, schema, data, folders_internal)
nc_global = netcdf.getConstant('GLOBAL');
if exist('folders_internal', 'var') && ~isempty(folders_internal)
    grp_id = netcdf.inqNcid(nc_id, folders_internal{1});
    writeData(grp_id, schema, data, folders_internal(2:end));
    return
end
if isfield(schema, 'Attributes')
    for att_ref = schema.Attributes
        att_name = att_ref.Name;
        value = att_ref.Value;
        if isempty(value)
            if isfield(data, att_name)
                value = data.(att_name);
            else
                error('write_ncfile:BadInputData',...
                    'Field ''%s'' not found at correct location.',att_name)
            end
        end
        if islogical(value)
            value = int8(value);
        end
        netcdf.putAtt(nc_id, nc_global, att_name, value);
    end
end
if isfield(schema, 'Variables')
    for var_ref = schema.Variables
        var_name = var_ref.Name;
        if isfield(data, var_name)
            value = data.(var_name);
        else
            error('write_ncfile:BadInputData',...
                'Field ''%s'' not found at correct location.', var_name)
        end
        var_id = netcdf.inqVarID(nc_id, var_name);
        netcdf.putVar(nc_id, var_id, value);
    end
end
for grp_ref = schema.Groups
    grp_id = netcdf.inqNcid(nc_id, grp_ref.Name);
    writeData(grp_id, grp_ref, data.(grp_ref.Name));
end
end

function schema = defineNc(nc_id, schema, data, dim_array, folders_internal)
if exist('dim_array', 'var') ~= 1 || isempty(dim_array)
   dim_map= containers.Map();
end
if exist('folders_internal', 'var') && ~isempty(folders_internal)
    grpNc_id = netcdf.defGrp(nc_id, folders_internal{1});
    defineNc(grpNc_id, schema, data, dim_array, folders_internal(1:end-1));
    return
end
if isfield(schema, 'Groups')
    for grp_ref = schema.Groups
        grpNc_id = netcdf.defGrp(nc_id, grp_ref.Name);
        defineNc(grpNc_id, grp_ref, data.(grp_ref.Name), dim_array);
    end
end
for dim_ref = schema.Dimensions
    L = dim_ref.Length;
    if isempty(L)
        L = length(data.(dim_ref.Name));
    end
    dim_map(dim_ref.Name) = netcdf.defDim(nc_id, dim_ref.Name, L);
end
for var_ref = schema.Variables
    varDim_ref = var_ref.Dimensions;
    if ~isempty(varDim_ref)
        varDim_ref = cell2mat(values(dim_map,{varDim_ref.Name}));
    end
    bool_defl = isempty(var_ref.DeflateLevel) == 0;
    var_id = netcdf.defVar(nc_id, var_ref.Name,...
        convertFormatId(var_ref.Datatype), varDim_ref);
    netcdf.defVarFill(nc_id, var_id, false, var_ref.FillValue);
    netcdf.defVarDeflate(nc_id,var_id,var_ref.Shuffle,bool_defl,...
    var_ref.DeflateLevel);
    if ~isempty(var_ref.ChunkSize) || bool_defl
        size_dim = size(data.(var_ref.Name));
        chunk_var = 'CHUNKED';
        b = 10;
        a = ceil(max(size_dim)/b);
        chunk_size = min([size_dim;...
            a*ones(1, length(size_dim))], [], 1);
    else
        chunk_var = 'CONTIGUOUS';
        chunk_size = [];
    end
    netcdf.defVarChunking(nc_id, var_id,chunk_var, chunk_size)
    for att_ref = var_ref.Attributes
        netcdf.putAtt(nc_id, var_id, att_ref.Name, att_ref.Value);
    end
end
end

function [nc_type] = convertFormatId(mat_type)
    switch mat_type
        case 'int8'
            nc_type = 'NC_BYTE';
        case 'int32'
            nc_type = 'NC_INT';
        case 'double'
            nc_type = 'NC_DOUBLE';
        case 'char'
            nc_type = 'NC_CHAR';
        otherwise
            error('Data type not recognized.')
    end
end
